VERSION = '0.0.57'
ROOT_DIR = '/root'
HOST_ADDRESS = 'eubflow.eubchain.com'
HTTP_ADDRESS = 'https://%s' % HOST_ADDRESS
WS_ADDRESS = 'wss://%s:54188/' % HOST_ADDRESS
ENABLE_VERBOSE = False
DOCKER_WHITE_LIST = ['rancher_agent']


from os import system

import click
from click import echo

import eubh.get as get_package
import eubh.put as put_package
import eubh.utils as utils
import eubh.watch as watch_package
from eubh.config import VERSION, HTTP_ADDRESS, WS_ADDRESS

_global_options = [
    click.option('--verbose', '-v', 'verbosity', flag_value=2, default=1, help='Enable verbose output'),
]


def global_options(func):
    for option in reversed(_global_options):
        func = option(func)
    return func


@click.group()
def get_group():
    pass


@get_group.command()
@click.argument('key', required=True)
@click.argument('path', required=True)
@click.option('--code/--no-code', default=True, help='Get code source')
@click.option('--result/--no-result', default=True, help='Get result')
@click.option('--unzip/--no-unzip', default=False, help='Unzip input')
@click.option('-ho', '--host', default=HTTP_ADDRESS, help="Server host")
def get(key, path, code, result, unzip, host):
    if code:
        get_package.Get(key).get(path, unzip, host)
    if result:
        get_package.Get(key, 'result').get(path, unzip, host)


@click.group()
def put_group():
    pass


@put_group.command()
@click.argument('key', required=True)
@click.argument('path', required=True)
@click.option('--folder/--no-folder', default=True, help='Upload directory')
@click.option('--code/--no-code', default=False, help='Upload code')
@click.option('--result/--no-result', default=False, help='Put result')
@click.option('-ho', '--host', default=HTTP_ADDRESS, help='Server host')
def put(key, path, folder, code, result, host):
    if code:
        put_package.Put(key).put(path, folder, host)
    if result:
        put_package.Put(key, 'result').put(path, folder, host)


@click.group()
def watch_group():
    pass


@watch_group.command()
@global_options
@click.option('-k', '--key')
@click.option('-t', '--time', default=10, help='Loop time (s)')
@click.option('-v', '--verbosity', default=True, help='Verbose mode')
@click.option('-ho', '--host', default=HTTP_ADDRESS, help='Watch server address')
@click.option('-w', '--webscoket', default=WS_ADDRESS, help='Webscoket server address')
@click.option('-m','--miner', default='', help='Miner docker cmd')
def watch(key, time, verbosity, host, webscoket, miner):
    watch_package.Watch(key, time, verbosity, host, webscoket, miner).watch()


@click.group()
def init_group():
    pass


@init_group.command()
def init():
    utils.init_environment()


@click.group()
def upgrade_group():
    pass


@upgrade_group.command()
def upgrade():
    system('pip install --upgrade eubh --no-cache-dir')


@click.group()
def version_group():
    pass


@version_group.command()
def version():
    echo(VERSION)


@click.group()
def container_group():
    pass


cli = click.CommandCollection(
    sources=[get_group, put_group, watch_group, init_group, upgrade_group, version_group])


def main():
    cli()


if __name__ == "__main__":
    cli()



import multiprocessing
import subprocess
import os
import signal
from uuid import getnode as get_mac

import docker
from apscheduler.schedulers.background import BlockingScheduler
from click import echo

import api
import get
import put
import status
import utils
from config import WS_ADDRESS, ROOT_DIR, DOCKER_WHITE_LIST, HTTP_ADDRESS
from logs.container_log import ContainerLog as WriteContainerLog
from wesockets.container_log import ContainerLog as PushContainerLog


def clean_file():
    echo('Remove %s/code and %s/result' % (ROOT_DIR, ROOT_DIR))
    os.popen('rm -rf %s/code %s/result' % (ROOT_DIR, ROOT_DIR))
    echo('Remove files successful.')


def create_directories():
    os.makedirs('%s/code' % ROOT_DIR)
    os.makedirs('%s/result' % ROOT_DIR)


def job_write_log(data, verbosity, host=HTTP_ADDRESS, webscoket=WS_ADDRESS):
    WriteContainerLog(pivot=data, verbosity=verbosity, host=host, webscoket=webscoket)


def job_real_time_push_log(data):
    push_container_log = PushContainerLog(WS_ADDRESS, data)

    try:
        push_container_log.connect()
    except KeyboardInterrupt:
        push_container_log.close()

def job_start_miner(cmd):
    if not utils.is_empty(cmd):
        try:
            p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)
            return p.pid
        except:
            echo('Running minner fail.')
            return 0
    else:
        return 0

class Watch:
    def __init__(self, key, time, verbosity, host, webscoket, miner):
        self.time = time
        self.key = key
        self.host = host
        self.webscoket = webscoket
        self.miner=miner
        self.api = api.Api(base_url=host)
        print("Api server:%s ,Webscoket:%s" % (webscoket, host))
        self.docker_client = docker.from_env()
        self.can_init = True
        self.cache_log = []
        self.jobs = []
        self.is_verbose = True if verbosity else False
        self.pid=0

    # def close_all_jobs(self):
    #     for job in self.jobs:
    #

    def clean_docker(self):
        client = self.docker_client
        for container in client.containers.list():
            if container.attrs['Name'] not in DOCKER_WHITE_LIST:
                container.stop()

    def run_cmd_and_upload_result(self, pivot):
        cmd = pivot.get('cmd')
        if not utils.is_empty(cmd):
            cmd_result_out_put = os.popen(cmd)
            cmd_result_out_put_string = cmd_result_out_put.read()
            self.api.project_machine_update_cmd({"id": pivot.get('id'), "result": cmd_result_out_put_string})
            cmd_result_out_put.close()

    def upload_machine_information(self):
        client = self.docker_client
"""
        current_status = status.Status.IDLE.value

        client_containers_count = len(client.containers.list())
        if client_containers_count == 0:
            if not utils.is_empty(self.key):
                current_status = status.Status.COMPLETE.value
            #???
            else:
                current_status = status.Status.IDLE.value

        if client_containers_count > 0:
            if utils.is_empty(self.key):
                self.clean_docker()
                current_status = status.Status.IDLE.value
            else:
                current_status = status.Status.RUNNING.value
"""

        client_containers_count = len(client.containers.list())
        if utils.is_empty(self.key):
            current_status = status.Status.IDLE.value
            if client_containers_count > 0:
                self.clean_docker()

        elif client_containers_count == 0:
            current_status = status.Status.COMPLETE.value
        elif client_containers_count > 0:
            current_status = status.Status.RUNNING.value

        data = utils.get_device_info()
        data['status_info'] = {
            'key': self.key,
            'status': current_status
        }
        data['mac'] = "machine_%s" % get_mac()
        response = self.api.upload_machine_and_get_task(data)

        if self.is_verbose:
            print(data, response)

        if type(response) is not list:
            option = response.get('option')
            project = response.get('project')
            pivot = response.get('pivot')

            key = None
            if project is not None:
                key = project.get('key')

            if not utils.is_empty(key):

                if self.pid != 0:
                    os.kill(self.pid, signal.SIGKILL)
                    os.kill(self.pid + 1, signal.SIGKILL)
                    self.pid = 0

                if option == 'init' and self.can_init:
                    self.can_init = False
                    if len(self.jobs) > 0:
                        for job in self.jobs:
                            job.terminate()
                    clean_file()
                    create_directories()
                    get.Get(key).get(ROOT_DIR, True, self.host)
                    self.key = key
                    self.run_cmd_and_upload_result(pivot)
                    # real_time_push_log = multiprocessing.Process(target=job_real_time_push_log, args=(pivot,))
                    # real_time_push_log.start()
                    #
                    # self.jobs.append(real_time_push_log)

                    write_log = multiprocessing.Process(target=job_write_log,
                                                        args=(pivot, self.is_verbose, self.host, self.webscoket))
                    write_log.start()

                    self.jobs.append(write_log)

                elif option == 'cmd':
                    self.run_cmd_and_upload_result(pivot)

            if option == 'clean':
                self.clean_docker()
                if not utils.is_empty(self.key):
                    put.Put(self.key, 'result').put('%s/result' % ROOT_DIR, True, self.host)
                    self.key = ''
                    self.can_init = True

        elif self.pid == 0 and current_status == status.Status.IDLE.value:
            self.pid = job_start_miner(self.miner)


    def log_is_exist(self, container_id, log):
        cache_logs = self.cache_log
        is_exist_log = False
        for i in cache_logs:
            if cache_logs[i].container_id == container_id and cache_logs[i].log == log:
                is_exist_log = True
                break
        if not is_exist_log:
            self.cache_log.append({
                container_id: container_id,
                log: log
            })
        return is_exist_log

    def watch_upload_docker_log(self):
        client = self.docker_client

        for container in client.containers.list():
            log = container.logs()

            self.api.post_project_container_logs({
                'container_id': container.attrs['Id'],
                'log': log
            })

    def watch(self):
        scheduler = BlockingScheduler()
        scheduler.add_job(self.upload_machine_information, 'interval', seconds=self.time, max_instances=1)
        try:
            scheduler.start()
        except (KeyboardInterrupt, SystemExit):
            scheduler.shutdown()



# -*- encoding: UTF-8 -*-
from setuptools import setup, find_packages

from src.eubh.config import VERSION

setup(name='eubh',
      version=VERSION,
      description='eub client help',
      keywords='eub client help',
      author='eubchain',
      author_email='service@eubchain.com',
      packages=find_packages(),
      include_package_data=True,
      zip_safe=True,
      install_requires=[
          'click', 'requests', 'azure', 'netifaces', 'apscheduler', 'docker', 'rarfile', 'ws4py'
      ],
      entry_points={
          'console_scripts': [
              'eubh=src.__main__:main'
          ]
      }
      )



import json

import requests

import config


class Api:
    def __init__(self, key=None, base_url=config.HTTP_ADDRESS):
        self.key = key
        self.base_url = base_url
        self.headers = {'Content-Type': 'application/json'}

    def get_url(self, url):
        return "%s/%s" % (self.base_url, url)

    def upload_machine_and_get_task(self, data={}):
        return requests.post(self.get_url('project/machine/%s' % (data.get('mac'))), data=json.dumps(data),
                             headers=self.headers).json()

    def project_machine_update_cmd(self, data={}):
        return requests.post(self.get_url('project/machine/%s/update-cmd' % data.get('id')), data=data)

    def store_project_container_log(self, data={}):
        return requests.post(self.get_url('project/container-log'), data=data)

    def get_project_info_by_key(self):
        return requests.get(self.get_url('project/project/%s' % self.key), headers=self.headers).json()

    def get_project_container_lists(self, key):
        return requests.get(self.get_url('project/container/lists/%s' % key), headers=self.headers).json()

    def get_project_container_logs_by_container_id(self, container_id):
        return requests.get(self.get_url('project/container-log/%s/logs-by-container-id' % container_id)).json()

    def post_project_container_logs(self, data={}):
        return requests.post(self.get_url('project/container-log/%s/set' % data.get('container_id')), data=data)

    def get_config_by_key(self):
        return self.get_project_info_by_key()
